;               Automatically load the 2000 Access I/O Processor.
;
;
; This command file loads the 2000 Access I/O Processor operating system via a
; cross-load from the System Processor.  It is invoked by the SP command files
; to start the IOP and prepare it for data exchange across the Processor
; Interconnect, as follows:
;
;    hp2100 iopload.inc <startup> <cpu-model> <cpu-series>
;
; ...where <startup> is "GEN" or "BOOT" for system generation or system startup,
; respectively, <cpu-model> is the specific CPU model to simulate (2100, 21MX,
; or 21MX-E), and <cpu-series> is the general CPU type (2100 or 21MX).  The
; command file is never invoked directly by the user.


; Capture the I/O Processor startup log.

set -N CONSOLE LOG=iopload-%3.log


; Configure the I/O Processor.

do iopconfig.inc %2 %3


; Configure the multiplexer ports for local or dataset operation.
;
; Ports that are configured for datasets will automatically disconnect their
; associated Telnet sessions if they are not logged on within the timeout set
; by the PHONES command, as well as when the sessions are logged off.  Local
; port Telnet sessions are not disconnected.  As an example, set ports 0 and 1
; for dataset connection.  Additional ports may be specified below as desired.

set MUXL0 DATASET
set MUXL1 DATASET


; Load paper into the line printer if the system is starting.

if "%1" == "BOOT" attach -N LPT printer.txt


; Attach the terminal multiplexer to listen for Telnet connections on localhost
; port 1054.

if "%1" == "BOOT" attach -Q MUX 1054


; Connect the Processor Interconnect cables to the IOP.
;
; The code number here ("4020") is significant only in that the SP instance
; must use the same code number to ensure interprocessor communication.

attach -I IPL 4020


; Start the binary loader, which is used as the cross loader via the IPL device.
; The loader starts with a CLC 0,C that must execute before the SP begins to
; send data.  So we load the bootstrap into memory and then step through until
; we reach the SFS in the input loop.
;
; In hardware, an HP 21xx IOP uses the BBL (Basic Binary Loader), whereas an HP
; 1000 IOP uses the paper tape loader ROM (12992K).  In either case, the loader
; is configured to read from the lower processor interconnect select code.
;
; In simulation, we use the IPL device loader, which automatically uses the
; correct loader for the current CPU model (actually, it uses the 2000 Access
; modified BBL for 21xx CPUs).  This permits changing the CPU model above
; without having to redo this section.

echo
echo Starting the IOP binary loader.

deposit S 000000
load IPL

step 20


; Signal the SP that we are ready to load the IOP program.

echo
echo Signalling the SP that that the IOP is ready to receive its program.

set IPL SIGNAL


; Continue cross-loader execution.

echo
echo Receiving the IOP program.

go


; Verify that we had a successful cross-load.

assert T=102077

echo
echo IOP cross-loading completed successfully.


; Initialize the IOP.
;
; The IOP program must complete its initialization before the SP begins to
; communicate.  Otherwise, the system will appear to start up normally, but the
; terminal multiplexer will be unresponsive.
;
; Unfortunately, there is no objective way of determining when initialization
; has completed.  In hardware, the system operator presses RUN on the IOP and
; then presses RUN on the SP.  The inherent human delay in these operations
; ensures that the IOP has executed through its initialization procedure and
; reached the communication loop.
;
; We approximate this in simulation by running the IOP until it reaches the
; address of the start of the communication loop (.COM.) and then signaling the
; SP to proceed.  This address is inherently version-specific.
;
; A less-efficient alternate would be to STEP enough times to ensure that
; initialization is complete before signaling the SP.  This too is inherently
; version-specific; for the Rev. 1812 version supplied with the kit,
; initialization takes 266,345 instructions.

echo
echo Initializing the IOP.

deposit P 000002
go until 036434


; Signal the SP that initialization is complete.

echo
echo Signalling the SP that initialization is complete.

set IPL SIGNAL


; Resume IOP execution.
;
; The stop-on-I/O-error condition is enabled in the CPU before resuming.  When
; the SP exits, its IPL is detached, and this condition is detected as a cable
; break by our IPL device.  Normally, as in hardware, this condition goes
; unreported.  However, by setting the IOERR stop, simulator execution will halt
; with a "Cable not connected" error.  We use this to determine when to
; terminate the IOP instance.

echo
echo The IOP is up and running.
echo

set CPU STOP=IOERR

go


; When the SP exits, the IOP simulator stops, and it is safe for us to exit.

exit
